const assert = require('assert');
const path = require('path');
const child = require('./compiler');
const Oracle = require('./oracle');
const { tap, tapHtml } = require('./compat');

module.exports = class WebappWebpackPlugin {
  constructor(args) {
    const options = (typeof args === 'string') ? { logo: args } : args;
    assert(typeof options === 'object' && typeof options.logo === 'string', 'An input file is required');

    this.options = Object.assign({
      cache: true,
      inject: true,
      favicons: {},
      prefix: 'assets/',
    }, options);

    this.tags = {};
    this.tags.promise = new Promise((resolve, reject) => {
      this.tags.resolve = resolve;
      this.tags.reject = reject;
    });
    this.tags.promise.catch(() => { });
  }

  rule() {
    assert(path.isAbsolute(this.options.logo), '`logo` must be an absolute path');

    const rule = {
      include: path.resolve(this.options.logo),
      loader: require.resolve('./rule_loader'),
      options: { plugin: this },
    };

    return rule;
  }

  apply(compiler) {
    const oracle = new Oracle(compiler.context);

    {
      const {
        appName = oracle.guessAppName(),
        appDescription = oracle.guessDescription(),
        version = oracle.guessVersion(),
        developerName = oracle.guessDeveloperName(),
        developerURL = oracle.guessDeveloperURL(),
      } = this.options.favicons;

      Object.assign(this.options.favicons, {
        appName,
        appDescription,
        version,
        developerName,
        developerURL,
      });
    }

    if (typeof this.options.inject !== 'function') {
      const { inject } = this.options;
      this.options.inject = htmlPlugin =>
        inject === 'force'
        || htmlPlugin.options.favicons !== false && htmlPlugin.options.inject && inject;
    }

    tap(compiler, 'make', 'WebappWebpackPlugin', (compilation, callback) =>
      // Generate favicons
      child.run(this.options, compiler.context, compilation)
        .then(tags => {
          this.tags.resolve(tags);

          // Hook into the html-webpack-plugin processing and add the html
          tapHtml(compilation, 'WebappWebpackPlugin', (htmlPluginData, callback) => {
            if (this.options.inject(htmlPluginData.plugin)) {
              const idx = (htmlPluginData.html + '</head>').search(/<\/head>/i);
              htmlPluginData.html = [htmlPluginData.html.slice(0, idx), ...tags, htmlPluginData.html.slice(idx)].join('');
            }
            return callback(null, htmlPluginData);
          });
          return callback();
        })
        .catch(err => {
          this.tags.reject(err);
          return callback(err);
        })
    );
  }
}
